@@ -225,7 +225,7 @@ pygi_signal_closure_marshal(GClosure *closure,
     while (list_item) {
         PyObject *item = list_item->data;
         if (item->ob_refcnt > 1) {
-            _pygi_boxed_copy_in_place ((PyGIBoxed *)item);
+            pygi_boxed_copy_in_place ((PyGIBoxed *)item);
         }
         list_item = g_slist_next (list_item);
     }
