@@ -5,7 +5,7 @@
 #include "test-unknown.h"
 #include "test-floating.h"
 
-#include <pyglib-python-compat.h>
+#include "pygi-python-compat.h"
 
 static PyObject * _wrap_TestInterface__do_iface_method(PyObject *cls,
 						       PyObject *args,
@@ -29,8 +29,8 @@ test_type_get_type(void)
 	type_info = (GTypeInfo *)g_new0(GTypeInfo, 1);
 	
         g_type_query(parent_type, &query);
-        type_info->class_size = query.class_size;
-        type_info->instance_size = query.instance_size;
+        type_info->class_size = (guint16)query.class_size;
+        type_info->instance_size = (guint16)query.instance_size;
 	
         gtype = g_type_register_static(parent_type,
 				       "TestType", type_info, 0);
@@ -596,8 +596,46 @@ _wrap_test_floating_and_sunk_get_instance_list (PyObject *self)
     return py_list;
 }
 
+
+static PyObject *
+_wrap_test_parse_constructor_args (PyObject *self, PyObject *args)
+{
+    char *arg_names[] = {"label", NULL};
+    char *prop_names[] = {"label", NULL};
+    GParameter params[1] = {{0}};
+    PyObject *parsed_args[1];
+    guint nparams = 0;
+
+    if (!PyArg_ParseTuple(args, "O", &(parsed_args[0])))
+        return NULL;
+
+    if (!pyg_parse_constructor_args (
+            TYPE_TEST, arg_names, prop_names, params, &nparams, parsed_args)) {
+        return NULL;
+    }
+
+    return PYGLIB_PyLong_FromLong (nparams);
+}
+
+static PyObject *
+_wrap_test_to_unichar_conv (PyObject * self, PyObject *args)
+{
+    PyObject *obj;
+    gunichar result;
+
+    if (!PyArg_ParseTuple(args, "O", &obj))
+      return NULL;
+
+    if (!pyg_pyobj_to_unichar_conv (obj, &result))
+        return NULL;
+
+    return PYGLIB_PyLong_FromLong (result);
+}
+
 static PyMethodDef testhelper_functions[] = {
+    { "test_parse_constructor_args", (PyCFunction)_wrap_test_parse_constructor_args, METH_VARARGS },
     { "get_test_thread", (PyCFunction)_wrap_get_test_thread, METH_NOARGS },
+    { "test_to_unichar_conv", (PyCFunction)_wrap_test_to_unichar_conv, METH_VARARGS },
     { "get_unknown", (PyCFunction)_wrap_get_unknown, METH_NOARGS },
     { "create_test_type", (PyCFunction)_wrap_create_test_type, METH_NOARGS },
     { "test_g_object_new", (PyCFunction)_wrap_test_g_object_new, METH_NOARGS },
